import GoodreadsSearch from "@components/GoodreadsSearch.svelte";
import { type SearchResult } from "@data-sources/goodreads";
import { Event, EventEmitter } from "@utils/event";
import { App, Modal, Notice } from "obsidian";
import { mount, unmount } from "svelte";

export class SearchEvent extends Event {
	constructor(
		public readonly query: string,
		public readonly results: SearchResult[]
	) {
		super();
	}
}

export class ErrorEvent extends Event {
	constructor(public readonly error: Error) {
		super();
	}
}

interface GoodreadsSearchModalEventMap {
	search: SearchEvent;
	error: ErrorEvent;
}

export class GoodreadsSearchModal extends EventEmitter<
	GoodreadsSearchModalEventMap,
	typeof Modal
>(Modal) {
	private component: ReturnType<typeof GoodreadsSearch> | undefined;

	onOpen() {
		this.component = mount(GoodreadsSearch, {
			target: this.contentEl,
			props: {
				onError: (error: Error) => {
					this.emit("error", new ErrorEvent(error));
				},
				onSearch: (query: string, results: SearchResult[]) => {
					this.emit("search", new SearchEvent(query, results));
				},
			},
		});
	}

	onClose() {
		if (this.component) {
			unmount(this.component);
			this.component = undefined;
		}
	}

	static createAndOpen(app: App): Promise<SearchResult[]> {
		return new Promise((resolve, reject) => {
			const modal = new GoodreadsSearchModal(app);
			modal.once("search", (event: SearchEvent) => {
				modal.close();
				resolve(event.results);
			});
			modal.once("error", (event: ErrorEvent) => {
				modal.close();
				reject(event.error);
			});
			modal.open();
		});
	}
}
