import Rating from "@components/Rating.svelte";
import { Event, EventEmitter } from "@utils/event";
import { App, Modal } from "obsidian";
import { mount, unmount } from "svelte";

class SubmitEvent extends Event {
	constructor(public readonly rating: number) {
		super();
	}
}

interface RatingModalEventMap {
	submit: SubmitEvent;
}

export class RatingModal extends EventEmitter<
	RatingModalEventMap,
	typeof Modal
>(Modal) {
	private component: ReturnType<typeof Rating> | undefined;

	onOpen(): void {
		this.component = mount(Rating, {
			target: this.contentEl,
			props: {
				onSubmit: (rating: number) =>
					this.emit("submit", new SubmitEvent(rating)),
			},
		});
	}

	onClose(): void {
		if (this.component) {
			unmount(this.component);
			this.component = undefined;
		}
	}

	static createAndOpen(app: App): Promise<number> {
		return new Promise((resolve) => {
			const modal = new RatingModal(app);
			modal.once("submit", (event: SubmitEvent) => {
				modal.close();
				resolve(event.rating);
			});
			modal.open();
		});
	}
}
