import ReadingLogEntryEditor from "@components/ReadingLogEntryEditor.svelte";
import type { ReadingLogEntry } from "@src/types";
import { Event, EventEmitter } from "@utils/event";
import { App, Modal } from "obsidian";
import { mount, unmount } from "svelte";

class SubmitEvent extends Event {
	constructor(public readonly entry: ReadingLogEntry) {
		super();
	}
}

interface ReadingLogEntryEditModalEventMap {
	submit: SubmitEvent;
}

export class ReadingLogEntryEditModal extends EventEmitter<
	ReadingLogEntryEditModalEventMap,
	typeof Modal
>(Modal) {
	private component: ReturnType<typeof ReadingLogEntryEditor> | undefined;

	constructor(app: App, private readonly entry: ReadingLogEntry) {
		super(app);
	}

	onOpen(): void {
		this.component = mount(ReadingLogEntryEditor, {
			target: this.contentEl,
			props: {
				entry: this.entry,
				onSubmit: (entry: ReadingLogEntry) =>
					this.emit("submit", new SubmitEvent(entry)),
			},
		});
	}

	onClose(): void {
		if (this.component) {
			unmount(this.component);
			this.component = undefined;
		}
	}
}
